import requests


def trigger_miqa_test(miqa_server, trigger_id, version_name):
    trigger_url = f"https://{miqa_server}/api/trigger_test_auto/{trigger_id}?app=mn&offline_version=True&name={version_name}"
    trigger_response_json = requests.get(trigger_url).json()
    run_id = trigger_response_json.get('run_id')
    return run_id


def get_tcr_info_json(miqa_server, run_id, directory_containing_outputs=None, ds_id=None, wfv_id=None):
    get_info_url = f"https://{miqa_server}/api/get_tcr_exec_info/{run_id}"
    query_pars = []
    if directory_containing_outputs:
        query_pars.append(f"source_location={directory_containing_outputs}")
    if ds_id:
        query_pars.append(f"ds_id={ds_id}")
    if wfv_id:
        query_pars.append(f"&wfv_id={wfv_id}")

    if len(query_pars)>0:
        get_info_url = f"{get_info_url}?{'&'.join(query_pars)}"

    get_info_response_json = requests.get(get_info_url).json()
    return get_info_response_json


def get_trigger_info(miqa_server, trigger_id):
    trigger_url = f"https://{miqa_server}/api/test_trigger/{trigger_id}/get_ds_id_mapping"
    trigger_response_json = requests.get(trigger_url).json()
    # return {"ds_id_mapping":{"results":trigger_response_json, "url":trigger_url}}
    return {"ds_id_mapping":{"results":trigger_response_json}}


import asyncio
import os

import aiohttp
import requests
from aiohttp import FormData

from ..baseuploadhelpers import get_remote_url, cloud_upload


async def upload_folder_async(folder, server, bucket, subfolder="folder-up3", filepattern=None, filepattern_start=None, quiet=True, cloud_provider='google', org_config_id=None, exclude_filepattern_end=None, max_filesize=None, api_key=None, max_connections=None):
    if not quiet:
        print(f"Uploading folder {folder}")
    files = [os.path.join(dp, f) for dp, dn, fn in os.walk(os.path.expanduser(folder)) for f in fn if
             not f.endswith(".DS_Store")]
    max_files = len(files)
    sorted_files = sorted({k:os.stat(k).st_size for k in files if (not max_filesize or (os.stat(k).st_size/(1024*1024) < max_filesize))}.items(), key=lambda kv: kv[1])
    filesize_msg = f'(filtered out {(max_files - len(sorted_files))} by size)' if max_filesize else ''
    print(f"Processing up to {len(sorted_files)} files from this directory {filesize_msg}...")

    if max_connections:
        connector = aiohttp.TCPConnector(limit=max_connections)
        client = aiohttp.ClientSession(connector=connector)
    else:
        client = aiohttp.ClientSession()
    async with client as session:
        num_processed = 0
        num_skipped = 0
        for file, filesize in sorted_files:
            if (not filepattern or file.endswith(filepattern)) and (not filepattern_start or file.startswith(filepattern_start)) and (not exclude_filepattern_end or not file.endswith(exclude_filepattern_end)):
                print(f"{file}: {round(filesize/ (1024 * 1024),3)}MB")
                subsubfolder = "/".join(file[len(folder)+1:].split("/")[:-1])
                filepath = os.path.join(folder, file)
                if os.path.isdir(filepath):
                    num_skipped+=1
                    print(f"Skipping directory: {num_skipped} total skipped so far")
                    continue
                num_processed += 1
                remote_url = get_remote_url(bucket, filepath, server, subfolder+"/"+subsubfolder, cloud_provider=cloud_provider, org_config_id=org_config_id, api_key=api_key)
                async with session.get(remote_url) as response:
                    if not quiet:
                        print("Status:", response.status)
                        print("Content-type:", response.headers['content-type'])

                    if response.status != 200:
                        print(f"ERROR: Unable to post file: response status was {response.status}")
                        json_r = await response.json()
                        if json_r and json_r.get('message'):
                            print(json_r.get('message'))
                        print("------------------------------------")
                        return

                    json_r = await response.json()
                    if not quiet:
                        print(json_r)
                    new_url = json_r.get('url')
                    content_length = os.path.getsize(filepath)
                    payload = open(filepath, "rb")
                    headers = {
                        'Content-Length': f'{content_length}',
                        'Content-Type': 'text/plain'
                    }

                    if cloud_provider == 'google':
                        async with session.put(new_url, data=payload, headers=headers) as response:
                            if not quiet:
                                print("submitted")
                    else:
                        data = FormData()
                        for dk, dv in new_url.get('fields', {}).items():
                            data.add_field(dk, dv)
                        data.add_field('file', open(filepath, 'rb'))
                        async with session.post(new_url.get('url'), data=data) as response:
                            if not quiet:
                                print("submitted")


def upload_folder_sync(folder, server, bucket, subfolder="folder-up3", filepattern=None, filepattern_start=None, quiet=True, cloud_provider='google', org_config_id=None, exclude_filepattern_end=None, max_filesize=None, api_key=None, max_connections=None):
    print("Processing folder synchronously...")
    if not quiet:
        print(f"Uploading folder {folder}")
    files = [os.path.join(dp, f) for dp, dn, fn in os.walk(os.path.expanduser(folder)) for f in fn if
             not f.endswith(".DS_Store")]
    max_files = len(files)
    sorted_files = sorted({k:os.stat(k).st_size for k in files if (not max_filesize or (os.stat(k).st_size/(1024*1024) < max_filesize))}.items(), key=lambda kv: kv[1])
    filesize_msg = f'(filtered out {(max_files - len(sorted_files))} by size)' if max_filesize else ''
    print(f"Processing up to {len(sorted_files)} files from this directory {filesize_msg}...")

    num_processed = 0
    num_skipped = 0
    for file, filesize in sorted_files:
        if (not filepattern or file.endswith(filepattern)) and (not filepattern_start or file.startswith(filepattern_start)) and (not exclude_filepattern_end or not file.endswith(exclude_filepattern_end)):
            print(f"{file}: {round(filesize/ (1024 * 1024),3)}MB")
            subsubfolder = "/".join(file[len(folder)+1:].split("/")[:-1])
            filepath = os.path.join(folder, file)
            if os.path.isdir(filepath):
                num_skipped+=1
                print(f"Skipping directory: {num_skipped} total skipped so far")
                continue
            num_processed += 1
            remote_url = get_remote_url(bucket, filepath, server, subfolder+"/"+subsubfolder, cloud_provider=cloud_provider, org_config_id=org_config_id, api_key=api_key)
            response = requests.get(remote_url)
            if not quiet:
                print("Status:", response.status_code)
                print("Content-type:", response.headers['content-type'])

            if response.status_code != 200:
                print(f"ERROR: Unable to post file: response status was {response.status_code}")
                json_r = response.json()
                if json_r and json_r.get('message'):
                    print(json_r.get('message'))
                print("------------------------------------")
                return

            json_r = response.json()
            if not quiet:
                print(json_r)
            new_url = json_r.get('url')
            content_length = os.path.getsize(filepath)
            payload = open(filepath, "rb")
            headers = {
                'Content-Length': f'{content_length}',
                'Content-Type': 'text/plain'
            }

            if cloud_provider == 'google':
                response = requests.put(new_url, data=payload, headers=headers)
                if not quiet:
                    print("submitted")
            else:
                # data = FormData()
                # for dk, dv in new_url.get('fields', {}).items():
                #     data.add_field(dk, dv)
                # data.add_field('file', open(filepath, 'rb'))
                response = requests.post(new_url.get('url'), data=new_url.get('fields', {}), files={'file': open(filepath, 'rb')})
                if not quiet:
                    print("submitted")


def upload_files_or_folder(args_folder, server, bucket, key, cloud_provider, org_config_id, args_files=None,
                           filepattern=None, filepattern_start=None, exclude_filepattern_end=None,
                           max_filesize=None, api_key=None, max_connections=None):
    global loop
    if args_folder:
        if max_connections and max_connections <= 1:
            upload_folder_sync(args_folder, server, bucket, subfolder=key, cloud_provider=cloud_provider,
                                org_config_id=org_config_id, filepattern=filepattern, filepattern_start=filepattern_start, exclude_filepattern_end=exclude_filepattern_end, max_filesize=max_filesize, api_key=api_key, max_connections=max_connections)
        else:
            loop = asyncio.get_event_loop()
            loop.run_until_complete(
                upload_folder_async(args_folder, server, bucket, subfolder=key, cloud_provider=cloud_provider,
                                    org_config_id=org_config_id, filepattern=filepattern, filepattern_start=filepattern_start, exclude_filepattern_end=exclude_filepattern_end, max_filesize=max_filesize, api_key=api_key, max_connections=max_connections))
    elif args_files:
        cloud_upload(args_files, server, bucket, key, cloud_provider=cloud_provider, org_config_id=org_config_id, api_key=api_key)



from ..executionhelpers import get_exec_info
from ..uploadhelpers import upload_files_or_folder


def upload_files_or_folder_for_exec(exec_id, server, args_folder=None, args_files=None, filepattern_end=None, filepattern_start=None, exclude_filepattern_end=None, max_filesize=None, api_key=None, max_connections=None):
    exec_info = get_exec_info(exec_id, server)
    bucket = exec_info.get('bucket')
    key = exec_info.get('key')
    cloud_provider = exec_info.get('cloud_provider', 'aws')
    org_config_id = exec_info.get('org_config_id')
    upload_files_or_folder(args_folder, server, bucket, key, cloud_provider, org_config_id, args_files,
                           filepattern=filepattern_end, filepattern_start=filepattern_start,
                           exclude_filepattern_end=exclude_filepattern_end,
                           max_filesize=max_filesize, api_key=api_key,
                           max_connections=max_connections)



import requests


def get_ds_ids_by_name(miqa_server, pipeline_id):
    info = get_ds_ids_by_name_internal(miqa_server, pipeline_id)
    return info


def get_ds_ids_by_name_internal(miqa_server, pipeline_id):
    remote_url = f'https://{miqa_server}/api/get_ds_ids_by_name?pipeline_id={pipeline_id}'
    info = requests.get(remote_url).json()
    return info


def tag_ds_ids_by_name(miqa_server, pipeline_id, name, tag):
    info = get_ds_ids_by_name_internal(miqa_server, pipeline_id)
    source_ids_lookup = info.get('data',{})
    if source_ids_lookup and source_ids_lookup.get(name):
        tag_url = f"https://{miqa_server}/api/batch_tag_datasources/{tag}?inline=1&source_ids={source_ids_lookup.get(name)}"
        resp = requests.get(tag_url)
        return resp
    else:
        raise Exception("No matching datasource IDs found")


def tag_ds_ids_by_names(miqa_server, pipeline_id, names, tag, raise_if_names_not_found=False):
    info = get_ds_ids_by_name_internal(miqa_server, pipeline_id)
    source_ids_lookup = info.get('data',{})
    source_ids_for_names = [source_ids_lookup.get(name) for name in names]
    source_ids_for_names = [str(i) for i in source_ids_for_names if i]
    if source_ids_for_names and (not raise_if_names_not_found or len(source_ids_for_names) == len(names)):
        tag_url = f"https://{miqa_server}/api/batch_tag_datasources/{tag}?inline=1&source_ids={','.join(source_ids_for_names)}"
        resp = requests.get(tag_url)
        return resp
    else:
        raise Exception("Matching datasource IDs not found")


def batch_create_datasets_and_tag_or_group(miqa_server, org_id, pipeline_id, new_ds_names, tags, dsg_name=None):
    tag_url = f"https://{miqa_server}/api/batch_create_dataset_and_tag?org_id={org_id}&pipeline_id={pipeline_id}&names={','.join(new_ds_names)}"

    if tags:
        tag_url += f"&tags={','.join(tags)}"
    if dsg_name:
        tag_url += f"&create_dsg={dsg_name}"

    resp = requests.get(tag_url)
    return resp


import requests


def complete_exec(exec_id, facet_server=None):
    print(f"Completing execution {exec_id}")
    remote_url = f'https://{facet_server}/api/execution/{exec_id}/mark_complete'
    info = requests.get(remote_url).json()
    print(info.get('message'))
    return info


def get_exec_info(exec_id, facet_server=None):
    remote_url = f'https://{facet_server}/api/execution/{exec_id}'
    info = requests.get(remote_url).json()
    return info


def get_exec_url(exec_id, facet_server=None):
    return f'https://{facet_server}/execution/{exec_id}'



from argparse import ArgumentParser

from ..executionhelpers import complete_exec
from ..offlineexecution import upload_files_or_folder_for_exec
from ..triggertest_helpers import trigger_miqa_test, get_tcr_info_json


def trigger_test_and_upload_single_results(miqa_server, trigger_id, version_name, directory_containing_outputs, filepattern_end=None, filepattern_start=None, exclude_filepattern_end=None, max_filesize=None, api_key=None, max_connections=None):
    run_id = trigger_miqa_test(miqa_server, trigger_id, version_name)
    get_info_response_json = get_tcr_info_json(miqa_server, run_id, directory_containing_outputs)
    exec_id = get_info_response_json.get("exec_id")
    upload_files_or_folder_for_exec(exec_id, miqa_server, directory_containing_outputs, None, filepattern_end=filepattern_end,
                                    filepattern_start=filepattern_start, exclude_filepattern_end=exclude_filepattern_end, max_filesize=max_filesize, api_key=api_key,
                                    max_connections=max_connections)
    complete_exec(exec_id, miqa_server)


def trigger_test_and_upload_by_dsid(miqa_server, trigger_id, version_name, ds_lookup, filepattern_end=None, filepattern_start=None, exclude_filepattern_end=None, max_filesize=None, api_key=None, max_connections=None):
    run_id = trigger_miqa_test(miqa_server, trigger_id, version_name)
    for ds_id, directory_containing_outputs in ds_lookup.items():
        get_info_response_json = get_tcr_info_json(miqa_server, run_id, directory_containing_outputs, ds_id=ds_id)
        exec_id = get_info_response_json.get("exec_id")
        upload_files_or_folder_for_exec(exec_id, miqa_server, directory_containing_outputs, None, filepattern_end=filepattern_end,
                                        filepattern_start=filepattern_start, exclude_filepattern_end=exclude_filepattern_end, max_filesize=max_filesize, api_key=api_key,
                                        max_connections=max_connections)
        complete_exec(exec_id, miqa_server)


if __name__ == '__main__':
    parser = ArgumentParser(description='A command line tool for interacting with the Miqa API')
    parser.add_argument('--trigger_id', type=str, default=None, help='Trigger ID in Miqa')
    parser.add_argument('--server', type=str, default=None, help='Miqa Server URL')
    parser.add_argument('--version_name', type=str, default=None, help='Version Name to create (e.g. MyPipeline v1.0, or commit ID e.g. abc123de')
    parser.add_argument('--directory', type=str, default=None, help='Path to local directory containing files to upload')
    args = parser.parse_args()

    trigger_test_and_upload_single_results(args.server, args.trigger_id, args.version_name, args.directory)




from setuptools import setup, find_packages


setup(
    name='miqatools',
    version='1.1.2',
    license='GPLv3',
    author="Gwenn Berry",
    author_email='gwenn@magnalabs.co',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://www.magnalabs.co',
    keywords='miqa',
    install_requires=[
          'asyncio', 'aiohttp',
      ]
)


from argparse import ArgumentParser

from ..triggertest_helpers import trigger_miqa_test, get_tcr_info_json


def get_upload_command(miqa_server, run_id, directory_containing_outputs='$(pwd)'):
    get_info_response_json = get_tcr_info_json(miqa_server, run_id, directory_containing_outputs)
    upload_command = get_info_response_json.get("upload_command")
    return upload_command


def trigger_test_and_get_command(miqa_server, trigger_id, version_name, directory_containing_outputs='$(pwd)'):
    run_id = trigger_miqa_test(miqa_server, trigger_id, version_name)
    upload_command = get_upload_command(miqa_server, run_id, directory_containing_outputs)
    return upload_command


# Example usage
# upload_cmd = trigger_test_and_get_command("{your trigger ID}", "{your version}", "{your directory}")
# upload_cmd = trigger_test_and_get_command("c1890d2c", "offline_Aug11_1236", "/Users/gwennberry/Desktop/test-data/demo_vcf_S21")


# Use the above as a starting point if you want to embed in your own scripts, or run as a CLI program as demonstrated below.
# If you'd rather or require to skip the Docker piece altogether,
# ask us about direct upload API via python!
if __name__ == '__main__':
    parser = ArgumentParser(description='A command line tool for interacting with the Miqa API')
    parser.add_argument('--trigger_id', type=str, default=None, help='Trigger ID in Miqa')
    parser.add_argument('--server', type=str, default=None, help='Miqa Server URL')
    parser.add_argument('--version_name', type=str, default=None, help='Version Name to create (e.g. MyPipeline v1.0, or commit ID e.g. abc123de')
    parser.add_argument('--directory', type=str, default='$(pwd)', help='Path to local directory containing files to upload')
    args = parser.parse_args()

    if args.trigger_id and args.server and args.version_name:
        upload_cmd = trigger_test_and_get_command(args.server, args.trigger_id, args.version_name, args.directory)
        print(upload_cmd)
